[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# Content Feeds Data Dictionary client library for Python

[![API Version](https://img.shields.io/badge/api-v0.2-blue)]()
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.ContentFeedsDataDictionary/0.21.15)](https://pypi.org/project/fds.sdk.ContentFeedsDataDictionary/v/0.21.15)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

An API for interacting with CTS Data Dictionary.

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 0.2
- SDK version: 0.21.15
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.ContentFeedsDataDictionary==0.21.15
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.ContentFeedsDataDictionary==0.21.15
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.ContentFeedsDataDictionary
from fds.sdk.ContentFeedsDataDictionary.api import navigator_api
from fds.sdk.ContentFeedsDataDictionary.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.ContentFeedsDataDictionary.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.ContentFeedsDataDictionary.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.ContentFeedsDataDictionary.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = navigator_api.NavigatorApi(api_client)
    basic_search_request = BasicSearchRequest(None) # BasicSearchRequest | A list of search terms (each term must be at least two characters).

    try:
        # Returns the data items (along with the products they belong to) that contain at least one of the search terms as a substring of either their name or description.
        # example passing only required values which don't have defaults set
        api_response = api_instance.navigator_basic_search_post(basic_search_request)

        pprint(api_response)
    except fds.sdk.ContentFeedsDataDictionary.ApiException as e:
        print("Exception when calling NavigatorApi->navigator_basic_search_post: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Returns the data items (along with the products they belong to) that contain at least one of the search terms as a substring of either their name or description.
    #     api_response, http_status_code, response_headers = api_instance.navigator_basic_search_post_with_http_info(basic_search_request)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.ContentFeedsDataDictionary.ApiException as e:
    #     print("Exception when calling NavigatorApi->navigator_basic_search_post: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Returns the data items (along with the products they belong to) that contain at least one of the search terms as a substring of either their name or description.
    #     async_result = api_instance.navigator_basic_search_post_async(basic_search_request)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.ContentFeedsDataDictionary.ApiException as e:
    #     print("Exception when calling NavigatorApi->navigator_basic_search_post: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Returns the data items (along with the products they belong to) that contain at least one of the search terms as a substring of either their name or description.
    #     async_result = api_instance.navigator_basic_search_post_with_http_info_async(basic_search_request)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.ContentFeedsDataDictionary.ApiException as e:
    #     print("Exception when calling NavigatorApi->navigator_basic_search_post: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.ContentFeedsDataDictionary

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.ContentFeedsDataDictionary.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.ContentFeedsDataDictionary

configuration = fds.sdk.ContentFeedsDataDictionary.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.ContentFeedsDataDictionary

configuration = fds.sdk.ContentFeedsDataDictionary.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.ContentFeedsDataDictionary

configuration = fds.sdk.ContentFeedsDataDictionary.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/data-dictionary*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*NavigatorApi* | [**navigator_basic_search_post**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/NavigatorApi.md#navigator_basic_search_post) | **POST** /navigator/basic_search | Returns the data items (along with the products they belong to) that contain at least one of the search terms as a substring of either their name or description.
*NavigatorApi* | [**navigator_data_items_product_id_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/NavigatorApi.md#navigator_data_items_product_id_get) | **GET** /navigator/data_items/{Product_id} | Get the list of data items associated with a product for Navigator.
*NavigatorApi* | [**navigator_products_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/NavigatorApi.md#navigator_products_get) | **GET** /navigator/products | Get the list of products for Navigator.
*NavigatorApi* | [**navigator_sources_data_item_id_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/NavigatorApi.md#navigator_sources_data_item_id_get) | **GET** /navigator/sources/{DataItem_id} | Get a list of sources where the specified data item can be found, for each delivery method, filtered to the appearances of the given data item within the specified product.
*NavigatorApi* | [**navigator_sources_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/NavigatorApi.md#navigator_sources_get) | **GET** /navigator/sources | Get a list of sources by delivery method for the given product_id
*NavigatorApi* | [**navigator_table_fields_table_id_get**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/NavigatorApi.md#navigator_table_fields_table_id_get) | **GET** /navigator/table_fields/{Table_id} | Get the list of fields associated with a table for Navigator, and the code information for the specified data item, if applicable.


## Documentation For Models

 - [ApiEndpoint](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/ApiEndpoint.md)
 - [BasicSearchRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/BasicSearchRequest.md)
 - [BasicSearchResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/BasicSearchResponse.md)
 - [BasicSearchResponseResults](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/BasicSearchResponseResults.md)
 - [DataItem](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/DataItem.md)
 - [NavigatorSourcesResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/NavigatorSourcesResponse.md)
 - [Product](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/Product.md)
 - [ReferenceGroup](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/ReferenceGroup.md)
 - [Table](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/Table.md)
 - [TableCodeFieldInfoResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/TableCodeFieldInfoResponse.md)
 - [TableCodeInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/TableCodeInfo.md)
 - [TableField](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/ContentFeedsDataDictionary/v1/docs/TableField.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.ContentFeedsDataDictionary.apis and fds.sdk.ContentFeedsDataDictionary.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.ContentFeedsDataDictionary.api.default_api import DefaultApi`
- `from fds.sdk.ContentFeedsDataDictionary.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.ContentFeedsDataDictionary
from fds.sdk.ContentFeedsDataDictionary.apis import *
from fds.sdk.ContentFeedsDataDictionary.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

